--[[
	Player class
	Caveware Digital :: 2017
]]

local Player = {}
Player.__index = Player


--[[
	Create a game state
	OUTPUT: The newly created game state
]]
function Player:new(source)
	local player = {}
	setmetatable(player, self)

	player.x = source.x --or 200
	player.y = source.y --or 200
	player.w = 16
	player.h = 16
	player.color = "lightgrey"

	player.foot_x_offset = 4
	player.foot_y_offset = 0
	player.default_offset = 4

	player.speed = 80
	player.speed_init = 80
	player.step_int = 50
	player.timer = 50
	player.move_interval = 20

	player.alpha = 255
	player.fade_time = 50
	player.sneak = false

	player.got_ducky = false

	player.direction = {x = 0, y = 0}
	player.facing = 0
	player.to_face = 0
	player.step = 0
	player.moving = false
	player.waves = {}
	player.feet = newExtendedTable({})

	player.buttons = 0

	player.body	= love.physics.newBody(World, player.x + player.w / 2, player.y + player.h / 2, "dynamic")
	player.shape	= love.physics.newRectangleShape(player.x, player.y)
	player.fixture	= love.physics.newFixture(player.body, player.shape)


	player.image = gfx.boot_invert

	player.foot = love.graphics.newCanvas(6,16)
	love.graphics.setCanvas(player.foot)
		love.graphics.clear()
		love.graphics.setBlendMode("alpha")
        love.graphics.setColor(255, 255, 255)
        love.graphics.draw(player.image, 0, 0)
    love.graphics.setCanvas()
	return player
end

function Player:setDucky(state)
	self.got_ducky = (state or self.got_ducky)
end
function Player:gotDucky()
	return self.got_ducky
end

function Player:getPosition()
	return self.x, self.y
end

function Player:getCollisionShape()
	return {x = self.x, y = self.y, w = self.w, h = self.h}
end

function Player:getWavesCollision()
	local t = {}
	for i, v in ipairs(self.waves) do
		table.insert(t, v:getCollisionShape())
	end
	return t
end

--[[
	Performed to draw the game state
]]
function Player:draw()
	for i, v in ipairs(self.feet) do
		--will set the foot direction to the movement
		if (math.deg(v.facing)%180) > 80 and (math.deg(v.facing)%180) < 100 then
			self.foot_x_offset = 0
			self.foot_y_offset = self.default_offset
		elseif (math.deg(v.facing)%180) > 10 and (math.deg(v.facing)%180) < 170 then
			self.foot_x_offset = self.default_offset/2
			self.foot_y_offset = self.default_offset/2
		else
			self.foot_x_offset = self.default_offset
			self.foot_y_offset = 0
		end
		local alpha = v.alpha
		if self.sneak then
			alpha = v.alpha/2
		end

		--alternates steps to emulate right and left feet
		if v.step == 0 then
			love.graphics.setColor(255,255,255,alpha)
			love.graphics.draw(v.image, v.x+self.foot_x_offset, v.y+self.foot_y_offset, v.facing)
			if not self.moving then
				love.graphics.draw(v.image, v.x-self.foot_x_offset, v.y-self.foot_y_offset, v.facing)
			end
		else
			love.graphics.setColor(255,255,255,alpha)
			love.graphics.draw(v.image, v.x-self.foot_x_offset, v.y-self.foot_y_offset, v.facing)
			if not self.moving then
				love.graphics.draw(v.image, v.x+self.foot_x_offset, v.y+self.foot_y_offset, v.facing)
			end
		end
	end
	for i, v in ipairs(self.waves) do
		v:draw()
	end
end


--[[
	Performed when the game state is paused
]]
function Player:pause() end


--[[
	Performed when the game state is resumed
]]
function Player:resume() end


--[[
	Performed to start the game state
]]
function Player:start(options)

end


--[[
	Performed to destroy the game state (eg. moving from level-to-level)
]]
function Player:stop() end


--[[
	Performed to update the game state (each step it's active)
]]
function Player:update(dt)
	local input = InputManager
	if input:wasPressed("shift") then
		self.speed = self.speed/2
		self.step_int = self.step_int + 40
		self.sneak = true
	end
	if input:wasReleased("shift") then
		self.speed = self.speed*2
		self.step_int = self.step_int - 40
		self.sneak = false
	end

	if input:wasPressed("directions") then
		self.buttons = self.buttons + 1
	end
	if input:wasReleased("directions") then
		self.buttons = self.buttons -1
	end

	local direction_x = input:get("right") - input:get("left")
	local direction_y = input:get("down") - input:get("up")


	if input:isHeld("directions") then
		if self.timer >= self.step_int then
			self:findDirection(direction_x,direction_y)
			if input:isHeld("left") then
				self.x = self.x - self.move_interval - self.speed*dt
			elseif input:isHeld("right") then
				self.x = self.x + self.move_interval + self.speed*dt
			end

			if input:isHeld("up") then
				self.y = self.y - self.move_interval - self.speed*dt
			elseif input:isHeld("down") then
				self.y = self.y + self.move_interval + self.speed*dt
			end

			self.step = (self.step + 1)%2
			self:changeDirection(dt)
			self:makeFoot()
			table.insert(self.waves, Wave:new( {
				x = self.x,
				y = self.y,
				sneak = self.sneak,
				travelled_from_origin = 0,
				color = self.color,
				max_dist = 120
			} ))
			for key, wave in ipairs(self.waves) do
				for k, particle in ipairs(wave) do
					particle.body:setLinearVelocity(particle.speed * math.cos(particle.theta), particle.speed * math.sin(particle.theta))
				end
			end
			self.timer = 0
			self:alphaReset()
		else
			self.timer = self.timer + 40*dt
		end
	end
	--[[removes feet that have become invisible from table
		but keeps 1 in table to indicate player when stopped
	]]
	local remove_feet = {}
	for i , v in ipairs(self.feet) do
		if #self.feet > 1 then
			v.alpha = v.alpha - self.fade_time*dt
			self.moving = true
		else
			if v.alpha > 80 then
				v.alpha = v.alpha - self.fade_time*dt
			else
				if v.alpha > 30 then
					v.alpha = v.alpha - self.fade_time*dt
				end
				self.moving = false
			end
		end

		if v.alpha <= 5 then
			table.insert(remove_feet, i)
		end
	end
	self.feet:remove(remove_feet)


	--updates sound waves
	for i, v in ipairs(self.waves) do
		v:update(dt)
	end
end

--sets transparency if sneaking
function Player:alphaReset()
	if self.sneak then
		self.alpha = 150
	else
		self.alpha = 255
	end
end

--finds the correct facing for the player
function Player:findDirection(dirX,dirY)
	local num_button = (self.buttons or 0)
	if num_button > 0 and num_button < 3 then
		self.to_face = math.angle(0, 0, dirX/num_button, dirY/num_button) + math.rad(90)
		if num_button == 2 then
			self.move_interval = 10
		elseif num_button == 1 then
			self.move_interval = 20
		end
	end
end

--changes facing direction to the desired over dt
function Player:changeDirection(dt)
	--turns the player towards the new direction of player
	if self.to_face ~= self.facing then
		local a = math.deg(self.to_face - self.facing)

		if (a > 180) then
		  a = a - 360
		elseif (a < -180) then
		  a = a + 360
		end
		self.facing = self.facing + math.rad(a)*40*dt
	end
end

--creates a foot object
function Player:makeFoot()
	table.insert(self.feet,{
			image = self.foot,
			x = self.x,
			y = self.y,
			alpha = 255,
			facing = self.facing,
			step = self.step
		})
	end

return Player
