--[[
	The basic splash screen at the start of chikun Games products
	Caveware Digital :: 2017
]]


local current_folder = ...
local splash = GameState:new()


--[[
	Initialises the splash screen
	INPUT: options - Default settings for this state
]]
function splash:start(options)
	love.graphics.setBackgroundColor(51, 51, 51)

	-- Set defaults
	self._alpha = 1
	self._image = love.graphics.newImage(current_folder .. "/logo.png")
	self._image:setFilter('linear', 'linear', 4)
	self._next_state = options.next_state or "splash"

	sfx.startup:play()

	-- Generate the quads for the static effect
	self._quads = {}
	for i = 0, 19 do
		local quad = love.graphics.newQuad(0, i * 14.5, 1920, 14.5, 1920, 290)
		table.insert(self._quads, quad)
	end

	-- Static effect will last for 0.48 seconds
	self._static = 480

	-- Set a timer for the splash to fade out after 1.1 seconds
	TimerManager:start(1100, function ()
		self._fading_out = true
	end)
end


--[[
	Performed upon this state being popped
]]
function splash:stop()
	-- Kill everything about this state
	self._alpha = nil
	self._fading_out = nil
	self._image = nil
	self._next_state = nil
	self._quads = nil
	self._sound = nil
	self._static = nil
end


--[[
	Update the splash screen each loop
	INPUT: dt - Delta time
]]
function splash:update(dt)
	-- Decrease static at the regular rate
	self._static = math.max(self._static - 1000 * dt, 0)

	-- If the splash screen has passed 0.8 seconds, it will fade out
	if (self._fading_out) then
		-- Determine alpha based on a fading rate of 0.4 seconds
		self._alpha = self._alpha - dt / .4
		local val = 51 * self._alpha
		love.graphics.setBackgroundColor(val, val, val)

		-- If the screen has faded out completely, move to the next state
		if (self._alpha <= 0) then
			love.graphics.setBackgroundColor(0, 0, 0)
			local next_state = self._next_state
			StateManager:pop()
			StateManager:push(next_state)
		end
	end
end


--[[
	Draws the splash screen
]]
function splash:draw()
	-- Determine size of screen
	local width, height = Flags.GAME_WIDTH, Flags.GAME_HEIGHT

	-- Determine centre points of screen
	local centre_x, centre_y = width / 2, height / 2

	-- Scaling of the logo
	local scale = math.min(width / 1920, height / 1080)

	-- Set colour based on inner transparency
	love.graphics.setColor(255, 255, 255, self._alpha * 255)

	-- Offset drawing position for static bars based on scale
	love.graphics.translate(centre_x - 960 * scale, centre_y - 145 * scale)

	-- Iterate through static bars
	for key, quad in ipairs(self._quads) do
		-- Determine coordinates for the bar of static
		local x = math.random(self._static * 3) - self._static * 1.5
		local y = (key - 1) * 14.5 * scale

		-- Draw the static bar
		love.graphics.draw(self._image, quad, x, y, 0, scale)
	end

	-- Reset positioning
	love.graphics.origin()
end


return splash
