--[[
	Wave class: holds and manages the behavior of a group of particles
	Caveware Digital :: 2017
]]


local Wave = {}
Wave.__index = Wave


--[[
	Create a wave
	OUTPUT: The newly created wave
]]
function Wave:new(source)
	local wave = {}
	setmetatable(wave, self)
	wave.particle_quantity = 48
	wave.max_dist = source.max_dist
	wave.split_counter = 1 -- number of times split has occured
	wave.split_distance = 50 -- distance at which number of particles initially doubles

	wave.req_interpose = false
	wave.origin_x = source.x
	wave.origin_y = source.y
	wave.theta = 0 -- may introduce small random range here to avoid repetitive clipping effects if moving in a straight line
	wave.particles = newExtendedTable({})
	wave.color = source.color
	wave.sneak = source.sneak
	for i = 1, wave.particle_quantity do
		local p = Particle:new{
			x = source.x,
			y = source.y,
			x_o = wave.origin_x,
			y_o = wave.origin_y,
			theta = wave.theta,
			sneak = wave.sneak,
			color = wave.color,
			max_dist = wave.max_dist,
		}
		table.insert(wave.particles, p)
		wave.theta = wave.theta + 360 / wave.particle_quantity
	end

	return wave
end

function Wave:getCollisionShape()
	local t = {}
	for i, v in ipairs(self.particles) do
		table.insert(t, v:getCollisionShape())
	end
	return t
end

--[[
	Performed to draw the wave
]]
function Wave:draw()
	for i, v in ipairs(self.particles) do
		v:draw()
	end
end
--[[
	Performed when the game state is paused (do we need these?)
]]
function Wave:pause() end


--[[
	Performed when the game state is resumed
]]
function Wave:resume() end


--[[
	Performed to start the game state
]]
function Wave:start(options) end


--[[
	Performed to destroy the game state (eg. moving from level-to-level)
]]
function Wave:stop() end


--[[
	Performed to check for wave splitting
]]
function Wave:update(dt)
	for i, v in ipairs(self.particles) do
		local return_dist = v:update(dt) -- returns distance travelled
		if v.distance < v.max_dist then
			v.origin_distance = v.origin_distance + v.speed * dt
		--	print(v.alpha_scale .. "	" .. v.alpha .. "	".. v.alpha_max)
			v.alpha_scale = v.alpha_scale - (0.1 * v.speed * dt / v.max_dist)
			v.alpha = v.alpha * (v.alpha_scale or 0)
		else
			table.remove(self.particles, i)
			v.body:destroy()
		end
		if return_dist > self.split_distance * self.split_counter then

			if not self.req_interpose then
				self.req_interpose = true
			end
		end
	end
	if self.req_interpose then
		local j --iterate backwards through all particles, placing new entries below the first referenced index
		for j = #self.particles, 1, -1 do
			if j ~= #self.particles then
				self:interpose(j, self.particles[j], j+1, self.particles[j+1])
			else
				self:interpose(j, self.particles[j], 1, self.particles[1])
			end
		end
		--print(#self.particles)
		self.req_interpose = false
		self.split_counter = self.split_counter + 1
		self.particle_quantity = #self.particles
		self.split_distance = 80 - (36/self.split_counter)

	--	print ("split distance: " .. self.split_distance)
	--	print ("split counter: " .. self.split_counter)
	end
end

--[[
	Performed to place new particles between existing ones to draw consistent curves as radius expands
]]
function Wave:interpose(i1, p1, i2, p2)
	local newTheta = math.angleInterp(p1.theta, p2.theta)
	local x_o, y_o = p1:getOrigin()
	local x, y = p1:initialisePosition(newTheta)
	local newP = Particle:new {
		x = x,
		y = y,
		x_o = x_o,
		y_o = y_o,
		theta = newTheta,
		travelled_total = p1.distance,
		travelled_from_origin = p1.origin_distance,
		max_dist = p1.max_dist,
		color = p1.color,
		alpha = p1.alpha
	}
	p1.alpha = p1.alpha
	table.insert(self.particles, i1 + 1, newP)
end


return Wave
