--[[
	Extra mathematical functions
	Caveware Digital :: 2017
]]


-- Seed the random number generator
math.randomseed(os.time())


--[[
	Calculates the angle from one point to another.
	INPUT: x1 and y1 are the coordinates of the first point.
	       x2 and y2 are the coordinates of the second point.
	OUTPUT: The angle from point1 to point 2 in radians.
]]
function math.angle(x1, y1, x2, y2)
	return math.atan2(y2 - y1, x2 - x1)
end


--[[
	Constrains a value to a minimum and a maximum.
	INPUT: Three numbers, min, the value to be clamped, and the max.
	OUTPUT: If the value is smaller than min, returns min.
	        If the value is larger then max, returns max.
	        Otherwise returns value.
]]
function math.clamp(min, value, max)
	return math.max(min, math.min(max, value))
end


--[[
	Calculates the distance from one point to another.
	INPUT: x1 and y1 are the coordinates of the first point.
	       x2 and y2 are the coordinates of the second point.
	OUTPUT: The distance from point1 to point 2.
]]
function math.distance(x1, y1, x2, y2)
	return ((x2 - x1) ^ 2 + (y2 - y1) ^ 2) ^ 0.5
end


--[[
	Returns the value of an arg rounded to dec decimal places, 0 if omitted.
	INPUT: The value to be rounded and the number of decimal places to round to.
	OUTPUT: value rounded to dec decimal places.
	NOTE: Negative dec results in rounding behind the decimals
	      eg. math.round(4777, -2) == 4800
]]
function math.round(value, dec)
	dec = 10 ^ (dec or 0)
	return math.floor(value * dec + 0.5) / dec
end


--[[
	Returns the sign of a value.
	INPUT: value is the number to be checked.
	       prevent_zero will turn 0 into 1.
	OUTPUT: If value is smaller than 0, returns -1.
	        If value is larger than 0, returns 1.
	        If value is equal to 0, returns 0.
]]
function math.sign(value, prevent_zero)
	if (value == 0) then
		return prevent_zero and 1 or 0
	else
		return value > 0 and 1 or -1
	end
end

-- return the unit vector from point 1 to point 2
function math.normalise(x1, y1, x2, y2)
	local dx = x2 - x1
	local dy = y2 - y1
	local dist = math.distance(x1, y1, x2, y2)

	return {
		dx = dx / distance,
		dy = dy / distance
	}
end



function math.angleInterp(angle1, angle2) --angle 2 is always counterclockwise, ie 'larger'

	if angle2 == 0 then
		newAngle = (360 + angle1 + angle2)/ 2
	else
		newAngle = (angle1 + angle2)/ 2
	end
	return newAngle
end

-- return the dot product of two 2D vectors of the form {x, y}
function math.dot(a, b)
	assert(type(a) == "table" and type(b) == "table", "math.dot() expects two vectors")
	return (a[1] * b[1]) + (a[2] * b[2])
end

-- piecewise basic operations on vectors
function math.vector_piece(a, b, operation)
	assert(type(a) == "table" and type(b) == "table" and type(c) == "string", "vector multiplcation needs vectors!")
	local operations = {"add", "sub", "mult", "div"}
	local vector = {}
	if #a > #b then
		counter = #b
	else
		counter = #a
	end
	while counter do
		if operation == "add" then
			table.insert(vector, -1, (a[counter] + b[counter]) or a[counter] or b[counter]) -- or statements account for vectors of differing lengths
		elseif operation == "sub" then
			table.insert(vector, -1, (a[counter] - b[counter]) or a[counter] or b[counter])
		elseif operation == "mult" then
			table.insert(vector, -1, (a[counter] * b[counter]) or a[counter] or b[counter])
		elseif operation == "div" then
			table.insert(vector, -1, (a[counter] / b[counter]) or a[counter] or b[counter])
		end
	end
	return vector
end

function math.reflectNormal(velocity, normal) -- reflects the vector about the normal , be sure to pass them in the order required
	assert(type(velocity) == "table" and type(normal) == "vector", "this function requires vectors")
	velocity_normal		= math.vector_piece(normal, math.dot(velocity, normal), "mult")
	velocity_tangent	= math.vector_piece(velocity, velocity_normal, "sub")
	return math.vector_piece(velocity_tangent, velocity_normal, "sub")
end
