--[[
	Extra table functions and the ExtendedTable class
	Caveware Digital :: 2017
]]


--[[
	Clone table
	INPUT: Table to clone
	OUTPUT: Cloned, distinct table
]]
function table.clone(t)
	return {unpack(t)}
end


--[[
	Shuffles a given table
	INPUT: Table to shuffle
	OUTPUT: The modified table
]]
function table.shuffle(t)
	local j

	for i = #t, 2, -1 do
		j = math.random(i)
		t[i], t[j] = t[j], t[i]
	end

	return t
end



--[[
	The extended table is very close to the regular table, but functions can be
	called directly on the table and some functions are extended (eg. remove)
]]
local ExtendedTable = {}
ExtendedTable.__index = ExtendedTable

--[[
	Adds extended table properties to a given table
	INPUT: Table with properties
	OUTPUT: Same table
]]
function newExtendedTable(values)
	local t = values
	setmetatable(t, ExtendedTable)
	return t
end

--[[
	Shallow-copies this extended table
	OUTPUT: New extended table with same properties as this one
]]
function ExtendedTable:clone()
	return newExtendedTable(table.clone(self))
end

--[[
	Joins all elements in the extended table together, with a separator
	INPUT: Separator, range start and range end
	OUTPUT: Concatenated string
]]
function ExtendedTable:concat(separator, range_start, range_end)
	return table.concat(self, separator, range_start, range_end)
end

--[[
	Destroys this extended table
]]
function ExtendedTable:destroy()
	self = nil
end

--[[
	Returns value at position in extended table
	INPUT: Position
	OUTPUT: Self
]]
function ExtendedTable:get(position)
	return self[position]
end

--[[
	Inserts value into extended table
	INPUT: Position and value
	OUTPUT: Self
]]
function ExtendedTable:insert(position, value)
	if (not value) then
		table.insert(self, position)
	else
		table.insert(self, position, value)
	end

	return self
end

--[[
	Iterates over a given table and inserts values at end of extended table
	OUTPUT: Self
]]
function ExtendedTable:insertSingles(values)
	for key, value in ipairs(values) do
		self:insert(value)
	end

	return self
end

--[[
	Removes data from the extended table
	INPUT: A table of items, a function to filter by, or an individual item
	OUTPUT: Self
]]
function ExtendedTable:remove(data)
	if (type(data) == "function") then
		for i = #self, 1, -1 do
			if (data(self[i])) then
				self:remove(i)
			end
		end
	elseif (type(data) == "table") then
		table.sort(data, function (a, b)
			return a > b
		end)

		for key, value in ipairs(data) do
			self:remove(value)
		end
	else
		table.remove(self, data)
	end

	return self
end

--[[
	Remove item(s) by value
	INPUT: Value of item to remove
	OUTPUT: Self
]]
function ExtendedTable:removeByValue(value)
	self:remove(function (item)
		return item == value
	end)

	return self
end

--[[
	Sets the item at position to value
	INPUT: Position and value
	OUTPUT: Self
]]
function ExtendedTable:set(position, value)
	self[position] = value
	return self
end

--[[
	Shuffles the extended table
	OUTPUT: Self
]]
function ExtendedTable:shuffle()
	table.shuffle(self)
	return self
end

--[[
	Sorts the extended table
	INPUT: Function to sort by
	OUTPUT: Self
]]
function ExtendedTable:sort(sorting_function)
	table.sort(self, sorting_function)
	return self
end

--[[
	Unpacks the table
	OUTPUT: Unpacked table
]]
function ExtendedTable:unpack()
	return unpack(self)
end