--[[
	MapTool 3.0
	Caveware Digital :: 2017
]]


local MapTool = {}


function MapTool:load(path)
	-- Load map into a table
	local map_data = require(path)

	local new_map = newExtendedTable({
		-- Components
		layers = newExtendedTable({}),
		objects = newExtendedTable({}),
		quads = newExtendedTable({}),
		tilesets = newExtendedTable({}),

		-- Properties
		name = map_name,
		tile_w = map_data.tilewidth,
		tile_h = map_data.tileheight,
		w = map_data.width,
		h = map_data.height
	})


	for key, ext_tileset in ipairs(map_data.tilesets) do
		-- Our new tileset <3
		local tileset = newExtendedTable({
			gid = ext_tileset.firstgid,
			margin = ext_tileset.margin,
			spacing = ext_tileset.spacing,

			image = AssetManager:getFromPath(ext_tileset.image),
			image_w = ext_tileset.imagewidth,
			image_h = ext_tileset.imageheight,
			tile_w = ext_tileset.tilewidth,
			tile_h = ext_tileset.tileheight
		})

		-- Determine quad value range
		local repeat_x = math.ceil((tileset.image_w - tileset.spacing) /
			tileset.tile_w) - 1
		local repeat_y = math.ceil((tileset.image_h - tileset.spacing) /
			tileset.tile_h) - 1

		repeat_x = (tileset.image_w - tileset.margin * 2 + tileset.spacing) /
			(tileset.tile_w + tileset.spacing);
		local y_pos    = tileset.margin

		-- Start counting at current gid
		local gid_q = tileset.gid

		-- Interpret quads from the tileset
		for current_y = 0, repeat_y - 1 do

			-- Reset x starting position
			local x_pos = tileset.margin

			for current_x = 0, repeat_x - 1 do
				-- Add current quad to the table
				new_map.quads[gid_q] =
				love.graphics.newQuad(x_pos + (current_x * tileset.tile_w),
				                      y_pos + (current_y * tileset.tile_h),
				                      tileset.tile_w, tileset.tile_h,
				                      tileset.image_w, tileset.image_h)

				-- Increment x starting position
				x_pos = x_pos + tileset.spacing

				-- Increment quad number
				gid_q = gid_q + 1
			end

			-- Increment y starting position
			y_pos = y_pos + tileset.spacing
		end

		-- Add to the list of map tilesets
		table.insert(new_map.tilesets, tileset)
	end


	-- Load layers into tables
	for key, layer in ipairs(map_data.layers) do

		-- Create new table based on layer name
		local new_layer = newExtendedTable({
			data    = layer.data,
			name    = layer.name,
			type    = layer.type,
			vars    = layer.properties,
			w       = layer.width,
			h       = layer.height
		})

		-- Create table for objects
		local new_objects = { }

		-- If layer isn't tilelayer, import objects
		if (layer.type ~= "tilelayer") then
			for key, object in ipairs(layer.objects) do
				-- Get data from object's quad, if available
				local quad_vars = {}

				if (object.gid) then
					if (object.gid > 1000) then
						object.gid = nil
					else
						print(#new_map.quads)
						quad_vars = {new_map.quads[object.gid]:getViewport()}
					end
				end

				-- Add object to table
				new_objects[#new_objects+1] = newExtendedTable({
					name = object.name,
					type = object.type,
					gid  = object.gid,
					w    = (quad_vars[3] or object.width),
					h    = (quad_vars[4] or object.height),
					x    = object.x,
					y    = object.y - (quad_vars[4] or 0),
					ox   = object.x,
					oy   = object.y - (quad_vars[4] or 0),
					prop = object.properties
				})
			end
		end

		-- Add new layer to map_data
		if (layer.name == "fg") then
			new_map.fg = new_objects
		else
			new_layer.objects = new_objects
			table.insert(new_map.layers, new_layer)
		end
	end

	-- Return the newly created map
	return new_map
end


-- Draws the current or given map
function MapTool:draw(given_map)

	-- If map isn't given, default to current
	local map = given_map or self.current

	-- Draw all layers
	for key, layer in ipairs(map.layers) do
		-- If tile layer...
		if (layer.type == "tilelayer") then
			-- ...then draw all tiles in layer
			for key, tile in ipairs(layer.data) do
				if (tile > 0) then
					-- Calculated placement of tile
					local tmp_y = math.floor((key - 1) / map.w)
					local tmp_x = key - (tmp_y * map.w) - 1

					local quad_vars = {map.quads[tile]:getViewport()}

					-- If tile exists, draw it
					if (tile > 0) then
						love.graphics.draw(self:getImageFromGID(tile, map),
						                   map.quads[tile], tmp_x * map.tile_w,
						                   tmp_y * map.tile_h -
						                   quad_vars[4] + map.tile_h)
					end
				end
			end
		else
			-- ...draw all objects in layer...
			for key, object in ipairs(layer.objects) do
				-- ...if that's possible
				if (object.gid) then
					love.graphics.draw(self:getImageFromGID(object.gid, map),
					                   map.quads[object.gid], object.x, object.y)
				end
			end
		end
	end
end

--[[
	Draw the foreground layer
	INPUT:  Map to draw (if ommited, map.current)
]]
function MapTool:drawFG(given_map)
	local map = given_map or self.current

	if (map.fg) then
		-- Check for foreground layer
		for key, object in ipairs(map.fg) do
			-- Draw if possible
			if (object.gid) then
				love.graphics.draw(self:getImageFromGID(object.gid, map),
				                   map.quads[object.gid], object.x, object.y)
			end
		end
	end
end


-- Returns the image used by a certain gid
function MapTool:getImageFromGID(gid, given_map)
	-- If map isn't given, default to current
	local map = given_map or self.current

	-- Seek backwards through tilesets until desired image found
	local num = #map.tilesets
	while (gid < map.tilesets[num].gid) do
		num = num - 1
	end

	-- Return found image
	return map.tilesets[num].image
end


return MapTool
