--[[
	Controls and keeps states up to date
	Caveware Digital :: 2017
]]


local StateManager = {
	_list = newExtendedTable({})
}


--[[
	If a state is loaded, draw it
]]
function StateManager:draw()
	self:_forCurrentState(function (state)
		love.graphics.setColor(255, 255, 255)
		state:draw()
	end)
end


--[[
	Returns the currently active state
	OUTPUT: The currently active state
]]
function StateManager:getActiveState()
	local state_list = self._list

	if (#state_list > 0) then
		local state_name = state_list:get(#state_list)
		return States[state_name]
	end
end


--[[
	Stops the active state, destroys it, and resumes the one below if it exists
]]
function StateManager:pop()
	local state_list = self._list

	-- Pop the current state out - if no current state, stop function execution
	if (not self:_forCurrentState(function (state)
		state:stop()
		state_list:remove(#state_list)
	end)) then return nil end

	-- Resume the new current state if exists
	self:_forCurrentState(function (state)
		state:resume()
	end)
end


--[[
	Add a state to the stack
	INPUT: state_to_start - Key of the state to add to the stack
	       options - A table to pass to the state's start function
]]
function StateManager:push(state_to_start, options)
	-- Pause the current state, if exists
	self:_forCurrentState(function (state)
		state:pause()
	end)

	-- Add new state to the internal list
	local new_state = States[state_to_start]
	new_state:start(options or {})
	self._list:insert(state_to_start)
end


--[[
	Updates the current state if it exists
	INPUT: dt - Delta time
]]
function StateManager:update(dt)
	self:_forCurrentState(function (state)
		state:update(dt)
	end)
end


--[[
	:private
	Calls the callback function, passing the active state in as a parameter
	INPUT:  callback - Function to be Performed
	OUTPUT: Whether or not the function was called (boolean)
]]
function StateManager:_forCurrentState(callback)
	local active_state = self:getActiveState()

	if (active_state) then
		callback(active_state)
	end

	return active_state and true or false
end


return StateManager
