--[[
	Functions to delay the call of functions
	Caveware Digital :: 2017
]]


local TimerManager = {
	_id_counter = 0,
	_list = {}
}


--[[
	Check to see if a timer with the given ID exists
	INPUT:  id - ID of timer to check for
	OUTPUT: Whether or not the timer exists (boolean)
]]
function TimerManager:exists(id)
	return self:get(id) and true or false
end


--[[
	Returns the timer with the given ID
	INPUT:  id - The ID of the wanted timer
	OUTPUT: The table of the generated timer (table)
]]
function TimerManager:get(id)
	for key, timer in ipairs(self._list) do
		if (timer.id == id) then
			return timer, key
		end
	end
end


--[[
	Starts a one-off or looping timer
	INPUT:  time - How long in milliseconds until the call or loop happens
	        callback - Function to be called at end of timer or every loop
	        is_looping - Whether or not this timer will loop constantly
	OUTPUT: The ID of the timer which has been created (integer)
]]
function TimerManager:start(time, callback, is_looping)
	self._id_counter = self._id_counter + 1

	table.insert(self._list, {
		callback = callback,
		id = self._id_counter,
		is_looping = is_looping,
		period = time / 1000,
		time = time / 1000
	})

	return self._id_counter
end


--[[
	Stops the timer with the given ID
	INPUT:  id - The ID of the timer
	OUTPUT: The timer which was removed
]]
function TimerManager:stop(id)
	local timer, key = self:get(id)

	if (timer) then
		table.remove(self._list, key)
		return timer
	end
end


--[[
	Stops all timers which are currently running
]]
function TimerManager:stopAll()
	local list = self._list

	for i = #list, 1, -1 do
		table.remove(self._list, i)
	end
end


--[[
	Updates all timers and removes them / performs callback if they have expired
	INPUT:  dt - Delta time
]]
function TimerManager:update(dt)
	local list = self._list

	for i = #list, 1, -1 do
		local timer = list[i]
		timer.time = timer.time - dt

		while timer.time <= 0 do
			timer.time = timer.time + timer.period
			if (timer.callback() or not timer.is_looping) then
				table.remove(list, i)
				break
			end
		end
	end
end


return TimerManager
